#! /usr/bin/env python
from molmod.io.chk import dump_chk
from yaff import System, log
from quickff.io import read_abinitio
import sys

# This script will generate a MolMod CHK file containing coordinates, bonds,
# atom types and hessian. The input required is a VASP xml file as well as
# the atom type rules for atom type definition (see below). This script
# should be used as:
#
#       python xml_to_chk.py file.xml
#
# Running this script requires a working installation of MolMod, Yaff and 
# QuickFF. If you succesfully installed QuickFF, then MolMod and Yaff should
# alse be installed already.

#turn of Yaff logging to screen
log.set_level(log.silent)

#parse XML file name from command line
fn_xml = sys.argv[1]

#extract coordinates, cell vectors, masses, hessian, ... from xml
numbers, coords, energy, grad, hess, masses, rvecs, pbc = read_abinitio(fn_xml)

#Define Yaff system to allow for bond detection and atom type definition
system = System(numbers, coords, rvecs=rvecs, masses=masses)

#automatically detect the bonds using the yaff routine detect_bonds applicable
#to Yaff System instances
system.detect_bonds()

#define the atom type rules according to the ATSELECT language. For more info
#on ATSELECT, see https://molmod.github.io/yaff/ug_atselect.html . Below an
#example is shown for unfunctionalized MIL-53(Al).
#TODO: ADJUST THE RULES TO YOUR SYSTEM!!!
rules = [
    ('C_PC', '6  & =3%6'),
    ('C_PH', '6  & =2%6  & =1%1'),
    ('C_CA', '6  & =1%6  & =2%8'),
    ('O_CA', '8  & =1%13 & =1%6'),
    ('O_HY', '8  & =2%13 & =1%1'),
    ('H_PH', '1  & =1%6'),
    ('H_HY', '1  & =1%8'),
    ('AL'  , '13 & =6%8'),
    ('C_N', '6 & =2%6 & =1%7'),
    ('N', '7 & =2%8 & =1%6'),
    ('O_NO', '8 & =1%7')
]

#detect the atom types using the above defined rules
system.detect_ffatypes(rules)

#dump everything to a CHK file
sample = {
    'numbers'   : numbers,
    'masses'    : system.masses,
    'pos'       : coords,
    'rvecs'     : rvecs,
    'ffatypes'  : system.ffatypes,
    'ffatype_ids': system.ffatype_ids,
    'bonds'     : system.bonds,
    'hessian'   : hess,
}
dump_chk(fn_chk, sample)
