##! /bin/env python3

from molmod.units import *
from quickff.tools import boxqp
from quickff.cost import *
import numpy as np
import matplotlib.pyplot as plt
import math
import pandas as pd
# from numba import jit
np.set_printoptions(suppress=True, linewidth=np.inf)

file = 'MOF211energyscan_withglobal.xlsx'
sheet = 'MOF211Enerscan_chargevasp_chged'
df_alldata = pd.read_excel(file, sheet_name=sheet, header=1, usecols="A:AC", nrows=13, engine='openpyxl')
# df_DFT=pd.read_excel(file, sheet_name=sheet,header=1,usecols="A:H",nrows=13,engine='openpyxl')
# df_FF=pd.read_excel(file,sheet_name=sheet,header=1,usecols="O:Q",nrows=13,engine='openpyxl')
# df_otherdihedrals=pd.read_excel(file,sheet_name=sheet,header=17,usecols="B:C",nrows=13,engine='openpyxl')

config_angles_top = {"Dihedral_O65_Cn": 'Dihedral(O_65 C_CA C_PC C_N)',
                     "Dihedral_O64_Cn": 'Dihedral(O_64 C_CA C_PC C_N)',
                     "Dihedral_O65_Cphtop": 'Dihedral(O_65 C_CA C_PC C_PH)top',
                     "Dihedral_O64_Cphtop": 'Dihedral(O_64 C_CA C_PC C_PH)top'}

config_angles_bottom = {"Dihedral_O71_Cph1": 'Dihedral(O_71 C_CA C_PC C_17)1',
                        "Dihedral_O74_Cph1": 'Dihedral(O_74 C_CA C_PC C_17)1',
                        "Dihedral_O71_Cph2": 'Dihedral(O_71 C_CA C_PC C_13)2',
                        "Dihedral_O74_Cph2": 'Dihedral(O_74 C_CA C_PC C_13)2'}
"""Torsions rest values"""
#restangles_top = {"chi_0_O65_Cn": 48.12, 'chi_0_O64_Cn': -139.76, 'chi_0_O65_Cphtop': -134.22, 'chi_0_O64_Cphtop': 37.90}
#restangles_bottom = {'chi_0_O71_Cph1': -15.25, 'chi_0_O74_Cph1': 163.63, 'chi_0_O71_Cph2': 163.63, 'chi_0_O74_Cph2': -15.25}

# restangles_top = {"chi_0_O65_Cn": 40.81, 'chi_0_O64_Cn': -142.55, 'chi_0_O65_Cphtop': -131.75,'chi_0_O64_Cphtop': 44.89}
# restangles_bottom = {'chi_0_O71_Cph1': -14.35, 'chi_0_O74_Cph1': 165.08, 'chi_0_O71_Cph2': 165.08, 'chi_0_O74_Cph2': -14.35}
"""as per vasp"""
restangles_top = {"chi_0_O65_Cn": -40.81, 'chi_0_O64_Cn': 142.55,
                  'chi_0_O65_Cphtop':131.75,'chi_0_O64_Cphtop': -44.89}
restangles_bottom = {'chi_0_O71_Cph1': 14.35, 'chi_0_O74_Cph1': -165.08,
                     'chi_0_O71_Cph2': -165.08, 'chi_0_O74_Cph2': 14.35}

config_energies = {"Edft": 'Relative(111) KJ/mol', "Effcovalent": 'ValenceContri(111 excl OCCCph KJ/mol)',
                   "Effvdw": 'VdwContri(111 KJ/mol)', "Effei": 'Electrostatic(111 KJ/mol)'}
Rotated_angles=np.array(df_alldata['Rotated angle'])*deg
config_angles = [config_angles_top, config_angles_bottom]
restangles = [restangles_top, restangles_bottom]
"""conversion to atomic units"""
for l in range(len(config_angles)):
    for (key1, value1) in config_angles[l].items():
        config_angles[l][key1] = np.array(df_alldata[value1]) * deg
for (key2, value2) in config_energies.items():
    config_energies[key2] = np.array(df_alldata[value2]) * kjmol
for i in range(len(restangles)):
    for (key3) in restangles[i]:
        restangles[i][key3] = restangles[i][key3] * deg

# print (config_angles)
Efftotal = (config_energies['Effcovalent'] + config_energies['Effvdw'] + config_energies['Effei'])  # in atomic units
EOCCC = (config_energies['Edft'] - Efftotal)  # in atomic units #Yi
num_angles = len(config_angles_top['Dihedral_O65_Cn'])

"""Fitting to cosine function.  m is different torsions we want to fit, and i is different trajectories.
f_m(chi_i4=0.5(1-Cosine multy(chi_i_m-chi_0_m))"""


def one_cosine(angle_arr, angle_res, multy):
    f = np.zeros(len(angle_arr))
    for i in range(len(f)):
        f[i] = (0.5 * (1 - np.cos(multy * (angle_arr[i] - angle_res))))
    return f


def OHy_OCa_diff(torsion, order):  # OHy, Oca are different and also top bottom dihedrals are considered different
    D_im = np.zeros([num_angles, (order * torsion) + 1])
    D_im[:, 0] = 1.0
    D_im[:, 1] = 2 * one_cosine(config_angles['Dihedral_OHy_Cn'], restangles['chi_0_OHy_Cn'])
    D_im[:, 2] = 2 * one_cosine(config_angles['Dihedral_OCa_Cn'], restangles['chi_0_OCa_Cn'])
    D_im[:, 3] = 2 * one_cosine(config_angles['Dihedral_OHy_Cph1'], restangles['chi_0_OHy_Cph'])
    D_im[:, 4] = 2 * one_cosine(config_angles['Dihedral_OCa_Cph1'],
                                restangles['chi_0_OCa_Cph'])  ##assuming this torsion is same near nitro and away
    # D_im[:,5]=one_cosine(config_angles['Dihedral_OHy_Cph2'],restangles['chi_0_OHy_Cph'])
    # D_im[:,6]=one_cosine(config_angles['Dihedral_OCa_Cph2'],restangles['chi_0_OCa_Cph'])
    # D_im[:,5]=one_cosine(config_angles['Dihedral_OHy_Cphtop'],restangles['chi_0_OHy_Cphtop'])
    # D_im[:,6]=one_cosine(config_angles['Dihedral_OCa_Cphtop'],restangles['chi_0_OCa_Cphtop'])
    return (D_im)


def OHy_OCa_same(num_multy, torsion=2):  # Top bottom dihedrals are considered different, but Ohy Oca same
    D_im = np.zeros([num_angles, ((num_multy * torsion) + 1)])
    # print (np.shape(D_im))
    odd = [num for num in range(1, ((num_multy * torsion) + 1)) if num % 2 == 1]
    even = [num for num in range(1, ((num_multy * torsion) + 1)) if num % 2 == 0]
    # print (odd,even)
    D_im[:, 0] = 1.0
    for m, indx in zip(multiplicity, odd):
        for dihedral, rest in zip(config_angles_top.values(), restangles_top.values()):
            # print (dihedral,rest,m,indx)
            D_im[:, indx] = D_im[:, indx] + one_cosine(dihedral, rest, m)

    for m1, indx1 in zip(multiplicity, even):
        for dihedral1, rest1 in zip(config_angles_bottom.values(), restangles_bottom.values()):
            #print(dihedral1, rest1, m1, indx1)
            D_im[:, indx1] = D_im[:, indx1] + one_cosine(dihedral1, rest1, m1)
    """
    #example of what above for loops do for each multiplicity
    D_im[:,1]=one_cosine(config_angles_top['Dihedral_OHy_Cn'],restangles_top['chi_0_OHy_Cn'],2)+one_cosine(config_angles_top['Dihedral_OCa_Cn'],restangles_top['chi_0_OCa_Cn'],2)\
              +one_cosine(config_angles_top['Dihedral_OHy_Cphtop'],restangles_top['chi_0_OHy_Cphtop'],2)+one_cosine(config_angles_top['Dihedral_OCa_Cphtop'],restangles_top['chi_0_OCa_Cphtop'],2)

    D_im[:,2]=one_cosine(config_angles_bottom['Dihedral_OHy_Cph1'],restangles_bottom['chi_0_OHy_Cph'],2)+one_cosine(config_angles_bottom['Dihedral_OCa_Cph1'],restangles_bottom['chi_0_OCa_Cph'],2)\
              +one_cosine(config_angles_bottom['Dihedral_OHy_Cph2'],restangles_bottom['chi_0_OHy_Cph1'],2)+one_cosine(config_angles_bottom['Dihedral_OCa_Cph2'],restangles_bottom['chi_0_OCa_Cph1'],2)

    D_im[:,3]=one_cosine(config_angles_top['Dihedral_OHy_Cn'],restangles_top['chi_0_OHy_Cn'],3)+one_cosine(config_angles_top['Dihedral_OCa_Cn'],restangles_top['chi_0_OCa_Cn'],3)\
              +one_cosine(config_angles_top['Dihedral_OHy_Cphtop'],restangles_top['chi_0_OHy_Cphtop'],3)+one_cosine(config_angles_top['Dihedral_OCa_Cphtop'],restangles_top['chi_0_OCa_Cphtop'],3)

    D_im[:,4]=one_cosine(config_angles_bottom['Dihedral_OHy_Cph1'],restangles_bottom['chi_0_OHy_Cph'],3)+one_cosine(config_angles_bottom['Dihedral_OCa_Cph1'],restangles_bottom['chi_0_OCa_Cph'],3)\
              +one_cosine(config_angles_bottom['Dihedral_OHy_Cph2'],restangles_bottom['chi_0_OHy_Cph1'],3)+one_cosine(config_angles_bottom['Dihedral_OCa_Cph2'],restangles_bottom['chi_0_OCa_Cph1'],3)
    """
    return D_im, torsion


def OHy_OCatopbott_same(num_multy, torsion=1):  # Top bottom dihedrals are also same, but Ohy Oca same
    D_im = np.zeros([num_angles, ((num_multy * torsion) + 1)])
    print(np.shape(D_im))
    odd = [num for num in range(1, ((num_multy * torsion) + 1)) if num % 2 == 1]
    even = [num for num in range(1, ((num_multy * torsion) + 1)) if num % 2 == 0]
    # print (odd,even)
    D_im[:, 0] = 1.0

    for m, indx in zip(multiplicity, range(1, (num_multy * torsion) + 1)):
        for dihedral, rest, dihedral1, rest1 in zip(config_angles_top.values(), restangles_top.values(),
                                                    config_angles_bottom.values(), restangles_bottom.values()):
            # print (dihedral/deg,rest/deg,dihedral1/deg,rest1/deg,m,indx)
            D_im[:, indx] = D_im[:, indx] + one_cosine(dihedral, rest, m) + one_cosine(dihedral1, rest1, m)
    """
    #example of what above for loops do for each multiplicity
    D_im[:,1]=one_cosine(config_angles_top['Dihedral_OHy_Cn'],restangles_top['chi_0_OHy_Cn'],2)+one_cosine(config_angles_top['Dihedral_OCa_Cn'],restangles_top['chi_0_OCa_Cn'],2)\
              +one_cosine(config_angles_top['Dihedral_OHy_Cphtop'],restangles_top['chi_0_OHy_Cphtop'],2)+one_cosine(config_angles_top['Dihedral_OCa_Cphtop'],restangles_top['chi_0_OCa_Cphtop'],2)

    D_im[:,2]=one_cosine(config_angles_bottom['Dihedral_OHy_Cph1'],restangles_bottom['chi_0_OHy_Cph'],2)+one_cosine(config_angles_bottom['Dihedral_OCa_Cph1'],restangles_bottom['chi_0_OCa_Cph'],2)\
              +one_cosine(config_angles_bottom['Dihedral_OHy_Cph2'],restangles_bottom['chi_0_OHy_Cph1'],2)+one_cosine(config_angles_bottom['Dihedral_OCa_Cph2'],restangles_bottom['chi_0_OCa_Cph1'],2)
    """
    return D_im, torsion


def svd_nocond(D_im, torsion):
    """singular value decomposition"""
    U, sigma, V = np.linalg.svd(D_im)
    # print ("sigma",sigma)
    sigma_r = np.zeros((np.shape(D_im)))
    np.fill_diagonal(sigma_r, sigma)
    print("sigma_r", sigma_r)
    V_r = V  # in this case V0 is already 0 since all singular values (sigma) are non zero.
    U_r = U[:, 0:(num_multy * torsion) + 1]  # This depends on number of singular values
    UtransY = np.dot(U_r.transpose(),
                     EOCCC)  # Utranspose*Y matrix where Y is Edue to OCCC; #relevant for final K calculation
    sigma_r = np.diag(sigma)
    sigma_rinv = np.linalg.inv(sigma_r)  # relevant for final K calculation
    V_rtrans = np.transpose(V_r)  # relevant for final K calculation
    """Finally calculating force constants"""
    K_forceconstant = np.dot(V_rtrans, (np.dot(sigma_rinv, UtransY)))
    print("K in KJ/mol", K_forceconstant / kjmol, np.shape(K_forceconstant))
    return K_forceconstant


def minmcostfunc_nosvd(D_im, EOCCC, K_init, torsion):
    """boxqp minimises cost function of the form X=1/2*xT.A.x - B.x, for eqn D_im*K=EOCCC, cost function will be 1/2*xT*DimT*D*x-EOCCCT*Dim*x+0.5*EOCCCT*EOCCC
    Hence A=DimT * Dim and B=EOCCCT * Dim """
    A = np.dot(D_im.T, D_im)
    B = np.dot(EOCCC.T, D_im)
    print("running", K_init / kjmol)
    K, nit = boxqp(A, B, np.zeros((num_multy * torsion) + 1), np.inf * np.ones((num_multy * torsion) + 1), x0=K_init,
                   status=True)
    print(nit)
    print("K from boxqp", K / kjmol)
    return K

# np.zeros(((num_multy*torsion)+1))

def minmcostfunc_svd(D_im, EOCCC, K_init, torsion):
    """boxqp minimises cost function of the form X=1/2*xT.A.x - B.x, for eqn D_im*K=EOCCC, cost function will be 0.5*xT*DimT*D*x-EOCCCT*Dim*x+0.5*EOCCCT*EOCCC
    Hence A=DimT * Dim and B=EOCCCT * Dim """
    svd_rcond = 0.0
    A = np.dot(np.transpose(D_im), D_im)
    B = np.dot(np.transpose(EOCCC), D_im)
    # init=np.zeros(((num_multy*torsion)+1))
    init = K_init
    print("running", init / kjmol)
    # perform SVD (taken from estimate function)
    U, S, Vt = np.linalg.svd(A, full_matrices=True)
    mask = S / max(S) > svd_rcond
    a = np.diag(S[mask])
    b = np.dot(U.T, B)[mask]
    vtx0 = np.dot(Vt, init)[mask]
    vtlower = np.zeros(len(b), float)
    vtupper = np.inf * np.ones(len(b), float)
    vtx = boxqp(a, b, vtlower, vtupper, vtx0)
    x = np.dot(Vt.T[:, mask], vtx)

    print("K from svd and boxqp", x / kjmol)
    return x

def FF_fitted_energy(D_im, K_forceconstant, torsion):
    """FF fitted energy"""
    Eff_fitted = np.zeros([num_angles, (num_multy * torsion) + 1])
    for mul in range((num_multy * torsion) + 1):
        Eff_fitted[:, mul] = (K_forceconstant[mul] * D_im[:, mul])
    Eff_fitOCCC = np.sum(Eff_fitted, axis=1)

    Eff_fitfinal = Efftotal + Eff_fitOCCC
    idx = np.where(config_energies['Edft'] == 0)
    shift = 0 - Eff_fitfinal[idx]
    Eff_fitfinal = Eff_fitfinal + shift
    return Eff_fitfinal


"""" Calling functions """
# D_im=OHy_OCa_diff(torsion=4,num_multy)
multiplicity = [2,3]
num_multy = len(multiplicity)
"""Generating D_im matrix based on how many torsions are considered"""
#D_im,torr=OHy_OCa_same(num_multy,torsion=2)
D_im, torr = OHy_OCatopbott_same(num_multy, torsion=1)

"""Different methods used to get the force constants(K), ie svd(can be + or - K) or use boxqp from QuickFF (only + K) or use svd with boxqp """
Kconstant_svd_nocond = svd_nocond(D_im, torr)  # Calculating force constants with svd
Kconstant_nosvd_cond=minmcostfunc_nosvd(D_im,EOCCC,Kconstant_svd_nocond,torr) #calculating force constants with boxqp function of QuickFF
#Kconstant_svd_cond=minmcostfunc_svd(D_im,EOCCC,Kconstant_svd_nocond,torr)
Ener_FFfitted = FF_fitted_energy(D_im, Kconstant_nosvd_cond, torr)


plt.plot(Rotated_angles/deg, config_energies['Edft'] / kjmol, label='Ab Initio', marker='s')
plt.plot(Rotated_angles/deg, Ener_FFfitted / kjmol, label='FF_fitted', marker='o')
plt.plot(Rotated_angles/deg, Efftotal / kjmol, label='FF_beforefit', marker='s')

#print (config_angles_top['Dihedral_O65_Cn']/deg)
# plt.plot(config_angles_top['Dihedral_O65_Cn']/deg, config_energies['Edft'] / kjmol,label='DFT', marker='s')
# plt.plot(config_angles_top['Dihedral_O65_Cn']/deg, Ener_FFfitted / kjmol, label='FF_fitted', marker='o')
# plt.plot(config_angles_top['Dihedral_O65_Cn']/deg, Efftotal / kjmol, label='FF_beforefit', marker='s')


# plt.plot(Dihedral_OHy_Cn/deg,EOCCC/kjmol,label='OCCC ref')
plt.xticks(np.arange(-180, 190, 30))
plt.ylabel("Energy (KJ/mole)")
plt.xlabel("Rotation angle (degree)")
plt.legend(loc="best")
plt.tight_layout()
plt.show()
