##! /usr/bin/python3

from molmod.units import *
from yaff import *
import numpy as np
import h5py
from glob import glob
import sys
# This script will do a single point calculation of the energy, gradient and
# virial tensor in the geometry defined in the input CHK file.
#
#   python ysp.py
#
file_xyz=sys.argv[1]
initfile=sys.argv[2]
rules = [
    ('C_PC', '6  & =3%6'),
    ('C_PH', '6  & =2%6  & =1%1'),
    ('C_CA', '6  & =1%6  & =2%8'),
    ('O_CA', '8  & =1%13 & =1%6'),
    ('O_HY', '8  & =2%13 & =1%1'),
    ('H_PH', '1  & =1%6'),
    ('H_HY', '1  & =1%8'),
    ('AL'  , '13 & =6%8'),
    ('C_N', '6 & =2%6 & =1%7'),
    ('N', '7 & =2%8 & =1%6'),
    ('O_NO', '8 & =1%7')
]
#Initialize the system using a MolMod CHK file
#sys = System.from_file(file_xyz,rvecs=np.array([[6.649000,0,0],[0,13.320000,0],[0,0,16.382000]])*angstrom)
#sys.detect_bonds()
#sys.detect_ffatypes(rules)
#sys.set_standard_masses()
#print (len(system.bonds),system.bonds)
#sys.to_file(initfile)

system = System.from_file(initfile)

# If you want to proceed with a supercell, e.g. a 1x2x1 supercell, you can do
# this as follows:
system = system.supercell(2,1,1)

# Initialize the force field. The force field parameters will be read from
# all files of the form pars*.txt (* being a wild cart) in the current 
# directory. Furthermore, the non-bonding cutoff parameters (rcut, alpha_scale,
# gcut_scale, smooth_ei) are set here. We suggest to not alter their values.
ff = ForceField.generate(system, glob('pars*.txt'), rcut=15.0*angstrom, alpha_scale=3.2, gcut_scale=1.5, smooth_ei=True)

# compute the energy, gradient and virial tensor of the system in the current
# configuration:
gpos = np.zeros(system.pos.shape) #a Nx3 array with N the number of atoms in the sytem
vtens = np.zeros([3,3])
energy = ff.compute(gpos=gpos, vtens=vtens)

# The energy is now stored in the variable 'energy' (in atomic units, i.e. hartree)
# The gradient (i.e. minus the forces) is now stored in the numpy array 'gpos' (in atomic units, i.e. hartree/bohr)
# The virial tensor is now stored in the numpy array 'vtens' (in atomic units, i.e. hartree/bohr**3)
#with open('./energy.out', 'w') as output:
 #   output.write("energy"+str(energy))
print ("Single_point_energy",energy)
